@@ -462,6 +462,7 @@ static errcode_t check_chain(o2fsck_state *ost,
 			     struct ocfs2_chain_rec *chain,
 			     char *buf1,
 			     char *buf2,
+			     char *pre_cache_buf,
 			     int *chain_changed,
 			     ocfs2_bitmap *allowed,
 			     ocfs2_bitmap *forbidden)
@@ -471,6 +472,8 @@ static errcode_t check_chain(o2fsck_state *ost,
 	uint64_t blkno;
 	errcode_t ret = 0;
 	int depth = 0, clear_ref = 0;
+	int blocks_per_group = ocfs2_clusters_to_blocks(ost->ost_fs,
+							cs->cs_cpg);
 
 	verbosef("free %u total %u blkno %"PRIu64"\n", chain->c_free,
 		 chain->c_total, (uint64_t)chain->c_blkno);
@@ -491,13 +494,13 @@ static errcode_t check_chain(o2fsck_state *ost,
 				int was_set;
 				ocfs2_bitmap_test(allowed, blkno, &was_set);
 				if (was_set) {
-					ocfs2_bitmap_clear(allowed, blkno,
-							   &was_set);
+					o2fsck_bitmap_clear(allowed, blkno,
+							    &was_set);
 					mark_group_used(ost, cs, bg1->bg_blkno,
 							allowed != NULL);
 				} else if (forbidden)
-					ocfs2_bitmap_set(forbidden, blkno,
-							 &was_set);
+					o2fsck_bitmap_set(forbidden, blkno,
+							  &was_set);
 			} else
 				mark_group_used(ost, cs, bg1->bg_blkno,
 						allowed != NULL);
@@ -525,6 +528,15 @@ static errcode_t check_chain(o2fsck_state *ost,
 			 * the read below.. */
 		}
 
+		/*
+		 * Pre-cache the entire group.  Don't care about failure.
+		 * If it works, the following ocfs2_read_group_desc() will
+		 * get the block out of the cache.
+		 */
+		if (pre_cache_buf)
+			ocfs2_read_blocks(ost->ost_fs, blkno,
+					  blocks_per_group, pre_cache_buf);
+
 		ret = ocfs2_read_group_desc(ost->ost_fs, blkno, (char *)bg2);
 		if (ret == OCFS2_ET_BAD_GROUP_DESC_MAGIC) {
 			if (prompt(ost, PY, PR_CHAIN_LINK_MAGIC,
@@ -632,12 +644,13 @@ out:
 static errcode_t verify_chain_alloc(o2fsck_state *ost,
 				    struct ocfs2_dinode *di,
 				    char *buf1, char *buf2,
+				    char *pre_cache_buf,
 				    ocfs2_bitmap *allowed,
 				    ocfs2_bitmap *forbidden)
 {
 	struct chain_state cs = {0, };
 	struct ocfs2_chain_list *cl;
-	uint16_t i, max_count;
+	int i, max_count;
 	struct ocfs2_chain_rec *cr;
 	uint32_t free = 0, total = 0;
 	int changed = 0, trust_next_free = 1;
@@ -728,7 +741,13 @@ static errcode_t verify_chain_alloc(o2fsck_state *ost,
 	if (trust_next_free)
 		max_count = cl->cl_next_free_rec;
 
-	for (i = 0; i < max_count; i++) {
+	/*
+	 * We walk the chains backwards for caching reasons.  Basically,
+	 * at the end the last blocks we read will be the most recently
+	 * used in the cache.  We want that to be the first chains,
+	 * especially for the inode scan, which will read forwards.
+	 */
+	for (i = max_count - 1; i >= 0; i--) {
 		cr = &cl->cl_recs[i];
 
 		/* reset for each run */
@@ -736,8 +755,8 @@ static errcode_t verify_chain_alloc(o2fsck_state *ost,
 			.cs_chain_no = i,
 			.cs_cpg = cl->cl_cpg,
 		};
-		ret = check_chain(ost, di, &cs, cr, buf1, buf2, &changed,
-				  allowed, forbidden);
+		ret = check_chain(ost, di, &cs, cr, buf1, buf2, pre_cache_buf,
+				  &changed, allowed, forbidden);
 		/* XXX what?  not checking ret? */
 
 		if (cr->c_blkno != 0) {
@@ -764,12 +783,13 @@ static errcode_t verify_chain_alloc(o2fsck_state *ost,
 			 * we copy the last chain into the missing spot
 			 * instead of shifting everyone over a spot 
 			 * to minimize the number of chains we have to
-			 * update */
+			 * update.  we then reset i so that we can go
+			 * over that chain and fix bg_chain */
 			if (i < (cl->cl_next_free_rec - 1)) {
 				*cr = cl->cl_recs[cl->cl_next_free_rec - 1];
 				memset(&cl->cl_recs[cl->cl_next_free_rec - 1],
 					0, sizeof(struct ocfs2_chain_rec));
-				i--;
+				i++;
 			}
 
 			cl->cl_next_free_rec--;
@@ -886,10 +906,10 @@ static errcode_t verify_bitmap_descs(o2fsck_state *ost,
 	     i < cgs.cgs_cluster_groups; 
 	     i++, blkno = i * ocfs2_clusters_to_blocks(ost->ost_fs,
 						       cgs.cgs_cpg)) {
-		ocfs2_bitmap_set(allowed, blkno, NULL);
+		o2fsck_bitmap_set(allowed, blkno, NULL);
 	}
 
-	ret = verify_chain_alloc(ost, di, buf1, buf2, allowed, forbidden);
+	ret = verify_chain_alloc(ost, di, buf1, buf2, NULL, allowed, forbidden);
 	if (ret) {
 		com_err(whoami, ret, "while looking up chain allocator inode "
 			"%"PRIu64, (uint64_t)di->i_blkno);
@@ -1040,15 +1060,25 @@ errcode_t o2fsck_pass0(o2fsck_state *ost)
 {
 	errcode_t ret;
 	uint64_t blkno;
+	uint32_t pre_repair_clusters;
 	char *blocks = NULL;
+	char *pre_cache_buf = NULL;
 	struct ocfs2_dinode *di = NULL;
 	ocfs2_filesys *fs = ost->ost_fs;
 	ocfs2_cached_inode **ci;
 	int max_slots = OCFS2_RAW_SB(fs->fs_super)->s_max_slots;
-	int i, type;
+	int i, type, bitmap_retried = 0;
 
 	printf("Pass 0a: Checking cluster allocation chains\n");
 
+	/*
+	 * The I/O buffer is 3 blocks. We apportion our I/O buffer
+	 * thusly:
+	 *
+	 * blocks[0] is the allocator inode we're working on.
+	 * blocks[1] & blocks[2] are used to hold group descriptors
+	 *     in functions below this one.
+	 */
 	ret = ocfs2_malloc_blocks(fs->fs_io, 3, &blocks);
 	if (ret) {
 		com_err(whoami, ret, "while allocating block buffers");
@@ -1056,6 +1086,27 @@ errcode_t o2fsck_pass0(o2fsck_state *ost)
 	}
 	di = (struct ocfs2_dinode *)blocks;
 
+	/*
+	 * We also allocate a pre-cache buffer of 4MB for reading entire
+	 * suballocator groups.  Some blocksizes have smaller groups, but
+	 * none have larger (see
+	 * libocfs2/alloc.c:ocfs2_clusters_per_group()).  This allows
+	 * us to pre-fill the I/O cache; we're already reading the group
+	 * descriptor, so slurping the whole thing shouldn't hurt.
+	 *
+	 * If this allocation fails, we just ignore it.  It's a cache.
+	 */
+	o2fsck_reset_blocks_cached();
+	if (o2fsck_worth_caching(1)) {
+		ret = ocfs2_malloc_blocks(fs->fs_io,
+					  ocfs2_blocks_in_bytes(fs, 4 * 1024 * 1024),
+					  &pre_cache_buf);
+		if (ret)
+			verbosef("Unable to allocate group pre-cache "
+				 "buffer, %s\n",
+				 "ignoring");
+	}
+
 	ret = ocfs2_malloc0(max_slots * sizeof(ocfs2_cached_inode *), 
 			    &ost->ost_inode_allocs);
 	if (ret) {
@@ -1109,12 +1160,51 @@ errcode_t o2fsck_pass0(o2fsck_state *ost)
 		}
 	}
 
+retry_bitmap:
+	pre_repair_clusters = di->i_clusters;
 	ret = verify_bitmap_descs(ost, di, blocks + ost->ost_fs->fs_blocksize,
 				  blocks + (ost->ost_fs->fs_blocksize * 2));
 
 	if (ret)
 		goto out;
 
+	if (pre_repair_clusters != di->i_clusters) {
+		if (prompt(ost, PY, PR_FIXED_CHAIN_CLUSTERS,
+			   "Repair of global_bitmap changed the filesystem "
+			   "from %u clusters to %u clusters.  Trust "
+			   "global_bitmap?",
+			   pre_repair_clusters, di->i_clusters)) {
+			ost->ost_num_clusters = di->i_clusters;
+			fs->fs_clusters = di->i_clusters;
+			fs->fs_blocks = ocfs2_clusters_to_blocks(fs,
+							 fs->fs_clusters);
+			ret = o2fsck_state_reinit(fs, ost);
+			if (ret) {
+				com_err(whoami, ret, "while reinit "
+					"o2fsck_state.");
+				goto out;
+			}
+
+			/*
+			 * The reinit clears the bits found during the
+			 * scan of the global bitmap.  We need to go over
+			 * them again.  They really should come out clean
+			 * this time.  If they don't, we probably have
+			 * a serious problem.
+			 *
+			 * In an interactive run, the user can keep
+			 * retrying and abort when they give up.  In a
+			 * non-interactive mode, we can't loop forever.
+			 */
+			if (ost->ost_ask || !bitmap_retried) {
+				bitmap_retried = 1;
+				verbosef("Restarting global_bitmap %s\n",
+					 "scan");
+				goto retry_bitmap;
+			}
+		}
+	}
+
 	printf("Pass 0b: Checking inode allocation chains\n");
 
 	/* first the global inode alloc and then each of the node's
@@ -1144,7 +1234,7 @@ errcode_t o2fsck_pass0(o2fsck_state *ost)
 					 blocks + ost->ost_fs->fs_blocksize,
 					 blocks + 
 					 (ost->ost_fs->fs_blocksize * 2), 
-					 NULL, NULL);
+					 pre_cache_buf, NULL, NULL);
 
 		/* XXX maybe helped by the alternate super block */
 		if (ret)
@@ -1197,7 +1287,7 @@ errcode_t o2fsck_pass0(o2fsck_state *ost)
 					 blocks + ost->ost_fs->fs_blocksize,
 					 blocks + 
 					 (ost->ost_fs->fs_blocksize * 2), 
-					 NULL, NULL);
+					 pre_cache_buf, NULL, NULL);
 
 		/* XXX maybe helped by the alternate super block */
 		if (ret)
@@ -1205,6 +1295,8 @@ errcode_t o2fsck_pass0(o2fsck_state *ost)
 	}
 
 out:
+	if (pre_cache_buf)
+		ocfs2_free(&pre_cache_buf);
 	if (blocks)
 		ocfs2_free(&blocks);
 	if (ret)
