@@ -460,14 +460,6 @@ static errcode_t fix_dirent_filetype(o2fsck_state *ost,
 		goto check;
 	}
 
-	ret = ocfs2_bitmap_test(ost->ost_bad_inodes, dirent->inode, &was_set);
-	if (ret)
-		goto out;
-	if (was_set) {
-		expected_type = OCFS2_FT_UNKNOWN;
-		goto check;
-	}
-
 	ret = o2fsck_type_from_dinode(ost, dirent->inode, &expected_type);
 	if (ret)
 		goto out;
@@ -888,8 +880,7 @@ errcode_t o2fsck_pass2(o2fsck_state *ost)
 	if (dp)
 		dp->dp_dirent = ost->ost_fs->fs_sysdir_blkno;
 
-	o2fsck_dir_block_iterate(&ost->ost_dirblocks, pass2_dir_block_iterate, 
-			 	 &dd);
+	o2fsck_dir_block_iterate(ost, pass2_dir_block_iterate, &dd);
 	o2fsck_strings_free(&dd.strings);
 out:
 	if (dd.dirblock_buf)
