@@ -126,6 +126,7 @@ void corrupt_file(ocfs2_filesys *fs, enum fsck_type type, uint16_t slotnum)
 	case EXTENT_EB_INVALID:
 		func = mess_up_extent_block;
 		break;
+	case EXTENT_MARKED_UNWRITTEN:
 	case EXTENT_BLKNO_UNALIGNED:
 		func = mess_up_extent_record;
 		break;
@@ -252,7 +253,9 @@ void corrupt_file(ocfs2_filesys *fs, enum fsck_type type, uint16_t slotnum)
 	case INODE_INLINE_CLUSTERS:
 		func = mess_up_inline_inode;
 		break;
-	case DUPLICATE_CLUSTERS:
+	case DUP_CLUSTERS_CLONE:
+	case DUP_CLUSTERS_DELETE:
+	case DUP_CLUSTERS_SYSFILE_CLONE:
 		func = mess_up_dup_clusters;
 		break;
 	default:
