@@ -59,6 +59,9 @@ static struct prompt_code prompt_codes[NUM_FSCK_TYPE] = {
 			   "so that fsck.ocfs2 can fix it"),
 	define_prompt_code(EXTENT_EB_INVALID, corrupt_file,
 			   "Corrupt an extent block's generation number"),
+	define_prompt_code(EXTENT_MARKED_UNWRITTEN, corrupt_file,
+			   "Mark an extent unwritten when the filesystem "
+			   "does not support it"),
 	define_prompt_code(EXTENT_BLKNO_UNALIGNED, corrupt_file,
 			   "Corrupt extent record's e_blkno"),
 	define_prompt_code(EXTENT_CLUSTERS_OVERRUN, corrupt_file,
@@ -149,8 +152,12 @@ static struct prompt_code prompt_codes[NUM_FSCK_TYPE] = {
 			   "Corrupt inlined inode's i_size"),
 	define_prompt_code(INODE_INLINE_CLUSTERS, corrupt_file,
 			   "Corrupt inlined inode's i_clusters"),
-	define_prompt_code(DUPLICATE_CLUSTERS, corrupt_file,
+	define_prompt_code(DUP_CLUSTERS_CLONE, corrupt_file,
 			   "Allocate same cluster to different files"),
+	define_prompt_code(DUP_CLUSTERS_DELETE, corrupt_file,
+			   "Allocate same cluster to different files"),
+	define_prompt_code(DUP_CLUSTERS_SYSFILE_CLONE, corrupt_file,
+			   "Allocate same cluster to different system files"),
 	define_prompt_code(CHAIN_COUNT, corrupt_sys_file,
 			   "Corrupt chain list's cl_count"),
 	define_prompt_code(CHAIN_NEXT_FREE, corrupt_sys_file,
