@@ -191,6 +191,7 @@ struct _ocfs2_devices {
 	int mount_flags;
 	int fs_type;			/* 0=unknown, 1=ocfs, 2=ocfs2 */
 	int hb_dev;
+	char stack[8];			/* Local, O2CB, CMAN, PCMK */
 	uint32_t maj_num;		/* major number of the device */
 	uint32_t min_num;		/* minor number of the device */
 	errcode_t errcode;		/* error encountered reading device */
@@ -560,6 +561,10 @@ errcode_t ocfs2_zero_tail_and_truncate(ocfs2_filesys *fs,
 				       ocfs2_cached_inode *ci,
 				       uint64_t new_size,
 				       uint32_t *new_clusters);
+errcode_t ocfs2_grow_chain_allocator(ocfs2_filesys *fs,
+				     int type,
+				     int slot_num,
+				     uint32_t num_clusters);
 errcode_t ocfs2_new_clusters(ocfs2_filesys *fs,
 			     uint32_t min,
 			     uint32_t requested,
