@@ -29,11 +29,10 @@
 #include <limits.h>
 #include <inttypes.h>
 
-#include "ocfs2.h"
+#include "ocfs2/ocfs2.h"
+#include "ocfs2/bitops.h"
 
-#include "bitops.h"
 #include "bitmap.h"
-#include "kernel-rbtree.h"
 
 
 /* The public API */
@@ -138,13 +137,21 @@ errcode_t ocfs2_bitmap_find_next_clear(ocfs2_bitmap *bitmap,
 }
 
 /* kind of poorly named, but I couldn't come up with something nicer */
-errcode_t ocfs2_bitmap_alloc_range(ocfs2_bitmap *bitmap, uint64_t len, 
-				   uint64_t *first_bit)
+errcode_t ocfs2_bitmap_alloc_range(ocfs2_bitmap *bitmap, uint64_t min,
+				   uint64_t len, uint64_t *first_bit,
+				   uint64_t *bits_found)
 {
-	if (len == 0 || len >= bitmap->b_total_bits)
+	errcode_t ret;
+
+	if (min == 0 || len == 0 || len >= bitmap->b_total_bits || min > len)
 		return OCFS2_ET_INVALID_ARGUMENT;
 
-	return (*bitmap->b_ops->alloc_range)(bitmap, len, first_bit);
+	ret = (*bitmap->b_ops->alloc_range)(bitmap, min, len, first_bit,
+					    bits_found);
+	if (ret == 0 && *bits_found < min)
+		abort();
+
+	return ret;
 }
 
 errcode_t ocfs2_bitmap_clear_range(ocfs2_bitmap *bitmap, uint64_t len, 
@@ -631,62 +638,85 @@ errcode_t ocfs2_bitmap_find_next_clear_generic(ocfs2_bitmap *bitmap,
 
 struct alloc_range_args {
 	ocfs2_bitmap	*ar_bitmap;
+	uint64_t	ar_min_len;
 	uint64_t	ar_len;
 	uint64_t	ar_first_bit;
+	uint64_t	ar_bits_found;
 	errcode_t	ar_ret;
 };
 
+/* Our strategy here to aid discontiguous allocation is to track the
+ * largest free regions (which still fit within ar_min_len) and if the
+ * max allocation fails, fall back to returning one of those. */
 static errcode_t alloc_range_func(struct ocfs2_bitmap_region *br,
 				  void *private_data)
 {
 	struct alloc_range_args *ar = private_data;
 	errcode_t ret = 0;
+	uint64_t best_start = UINT64_MAX, best_len = 0;
 	int start, end;
 
-	if ((br->br_total_bits - br->br_set_bits) < ar->ar_len)
+	if ((br->br_total_bits - br->br_set_bits) < ar->ar_min_len)
 		goto out;
 
-	for (start = 0; start + ar->ar_len <= br->br_total_bits; ) {
+	for (start = 0; start + ar->ar_min_len <= br->br_total_bits; ) {
 		start = ocfs2_find_next_bit_clear(br->br_bitmap,
 						  br->br_total_bits,
 						  start);
 		if (start == br->br_total_bits)
-			goto out;
+			break;
 
 		/* avoiding start + 1 here so that start at total_bits - 1
 		 * just works out */
 		end = ocfs2_find_next_bit_set(br->br_bitmap,
 					      br->br_total_bits,
 					      start);
-		if ((end - start) < ar->ar_len) {
-			start = end + 1;
-			continue;
+
+		/* We've found a region large enough to hold our max. */
+		if ((end - start) >= ar->ar_len) {
+			end = start + ar->ar_len;
+			goto found;
 		}
 
-		if ((end - start) > ar->ar_len)
-			end = start + ar->ar_len;
+		if ((end - start) > best_len) {
+			best_len = end - start;
+			best_start = start;
+		}
 
-		ar->ar_first_bit = br->br_start_bit + start;
-		for (; start < end; start++)
-			set_generic_shared(ar->ar_bitmap, br,
-					   start + br->br_start_bit);
-
-		ar->ar_ret = 0;
-		ret = OCFS2_ET_ITERATION_COMPLETE;
-		break;
+		start = end + 1;
 	}
 
+	/* Nothing found at all */
+	if (best_start == UINT64_MAX || best_len < ar->ar_min_len)
+		goto out;
+
+	/* Best fit works */
+	start = best_start;
+	end = best_start + best_len;
+found:
+	ar->ar_first_bit = br->br_start_bit + start;
+	ar->ar_bits_found = end - start;
+
+	for (; start < end; start++)
+		set_generic_shared(ar->ar_bitmap, br,
+				   start + br->br_start_bit);
+
+	ar->ar_ret = 0;
+	ret = OCFS2_ET_ITERATION_COMPLETE;
 out:
 	return ret;
 }
 
 errcode_t ocfs2_bitmap_alloc_range_generic(ocfs2_bitmap *bitmap,
+					   uint64_t min_len,
 					   uint64_t len,
-					   uint64_t *first_bit)
+					   uint64_t *first_bit,
+					   uint64_t *bits_found)
 {
 	errcode_t ret;
 	struct alloc_range_args ar = { 
 		.ar_bitmap = bitmap,
+		.ar_min_len = min_len,
 		.ar_len = len,
 		.ar_ret = OCFS2_ET_BIT_NOT_FOUND,
 	};
@@ -696,8 +726,10 @@ errcode_t ocfs2_bitmap_alloc_range_generic(ocfs2_bitmap *bitmap,
 	if (ret == 0)
 		ret = ar.ar_ret;
 
-	if (ret == 0)
+	if (ret == 0) {
 		*first_bit = ar.ar_first_bit;
+		*bits_found = ar.ar_bits_found;
+	}
 
 	return ret;
 }
@@ -714,7 +746,7 @@ errcode_t ocfs2_bitmap_clear_range_generic(ocfs2_bitmap *bitmap,
 		return OCFS2_ET_INVALID_BIT;
 
 	for (end = first_bit + len; first_bit < end; first_bit++)
-		clear_generic_shared(bitmap, br, first_bit + br->br_start_bit);
+		clear_generic_shared(bitmap, br, first_bit);
 
 	return 0;
 }
