@@ -233,6 +233,28 @@ static inline uint32_t system_dir_bytes_needed(State *s)
 	return each * sys_blocks_needed(s->initial_slots);
 }
 
+static void grow_extent_allocator(State *s, ocfs2_filesys *fs)
+{
+	errcode_t ret;
+	int i;
+
+	for (i = 0; i < OCFS2_RAW_SB(fs->fs_super)->s_max_slots; i++) {
+		ret = ocfs2_grow_chain_allocator(fs, EXTENT_ALLOC_SYSTEM_INODE,
+						 i, s->extent_alloc_size_in_clusters);
+		if (ret) {
+			com_err(s->progname, ret, "while growing the extent "
+				"allocator for slot %d by %d clusters",
+				i, s->extent_alloc_size_in_clusters);
+			goto error;
+		}
+	}
+	return;
+
+error:
+	clear_both_ends(s);
+	exit(1);
+}
+
 static void finish_normal_format(State *s)
 {
 	errcode_t ret;
@@ -271,6 +293,12 @@ static void finish_normal_format(State *s)
 	if (!s->quiet)
 		printf("done\n");
 
+	if (!s->quiet)
+		printf("Growing extent allocator: ");
+	grow_extent_allocator(s, fs);
+	if (!s->quiet)
+		printf("done\n");
+
 	if (!s->quiet)
 		printf("Formatting slot map: ");
 	format_slotmap(s, fs);
@@ -344,15 +372,20 @@ main(int argc, char **argv)
 
 	fill_defaults(s);
 
-	clear_both_ends(s);
-
 	generate_uuid (s);
 
 	create_generation(s);
 
 	print_state (s);
 
-	check_32bit_blocks (s);
+	check_32bit_blocks(s);
+
+	if (s->dry_run) {
+		close_device(s);
+		return 0;
+	}
+
+	clear_both_ends(s);
 
 	init_record(s, &superblock_rec, SFI_OTHER, S_IFREG | 0644);
 	init_record(s, &root_dir_rec, SFI_OTHER, S_IFDIR | 0755);
@@ -598,7 +631,7 @@ get_state(int argc, char **argv)
 	State *s;
 	int c;
 	int verbose = 0, quiet = 0, force = 0, xtool = 0, hb_dev = 0;
-	int show_version = 0;
+	int show_version = 0, dry_run = 0;
 	char *device_name;
 	int ret;
 	uint64_t val;
@@ -622,6 +655,7 @@ get_state(int argc, char **argv)
 		{ "heartbeat-device", 0, 0, 'H'},
 		{ "force", 0, 0, 'F'},
 		{ "mount", 1, 0, 'M'},
+		{ "dry-run", 0, 0, 'n' },
 		{ "no-backup-super", 0, 0, BACKUP_SUPER_OPTION },
 		{ "fs-feature-level=", 1, 0, FEATURE_LEVEL },
 		{ "fs-features=", 1, 0, FEATURES_OPTION },
@@ -636,7 +670,7 @@ get_state(int argc, char **argv)
 		progname = strdup("mkfs.ocfs2");
 
 	while (1) {
-		c = getopt_long(argc, argv, "b:C:L:N:J:M:vqVFHxT:", long_options,
+		c = getopt_long(argc, argv, "b:C:L:N:J:M:vnqVFHxT:", long_options,
 				NULL);
 
 		if (c == -1)
@@ -738,6 +772,10 @@ get_state(int argc, char **argv)
 			verbose = 1;
 			break;
 
+		case 'n':
+			dry_run = 1;
+			break;
+
 		case 'q':
 			quiet = 1;
 			break;
@@ -856,6 +894,7 @@ get_state(int argc, char **argv)
 	s->verbose       = verbose;
 	s->quiet         = quiet;
 	s->force         = force;
+	s->dry_run       = dry_run;
 
 	s->prompt        = xtool ? 0 : 1;
 
@@ -1188,6 +1227,41 @@ static uint32_t cluster_size_datafiles(State *s)
 	return cluster_size * 1024;
 }
 
+static uint32_t figure_extent_alloc_size(State *s)
+{
+	uint32_t cpg, numgroups;
+	uint64_t unitsize, totalsize;
+	double curr_percent, target_percent;
+
+	if (!s->initial_slots)
+		return 0;
+
+	switch (s->fs_type) {
+	case FS_DATAFILES:
+		target_percent = 0.3;
+		break;
+	case FS_MAIL:
+	default:
+		target_percent = 0.1;
+		break;
+	}
+
+	cpg = ocfs2_clusters_per_group(s->blocksize, s->cluster_size_bits);
+
+	/* size of the allocator across all slots with one group */
+	unitsize = cpg * s->cluster_size * s->initial_slots;
+
+	totalsize = unitsize;
+	for (numgroups = 1; ; ++numgroups) {
+		curr_percent = (double)totalsize * 100 / s->volume_size_in_bytes;
+		if (curr_percent >= target_percent)
+			break;
+		totalsize += unitsize;
+	}
+
+	return cpg * numgroups;
+}
+
 static void
 fill_defaults(State *s)
 {
@@ -1371,6 +1445,8 @@ fill_defaults(State *s)
 	}
 
 	s->journal_size_in_bytes = figure_journal_size(s->journal_size_in_bytes, s);
+
+	s->extent_alloc_size_in_clusters = figure_extent_alloc_size(s);
 }
 
 static int
@@ -2433,10 +2509,23 @@ static void
 print_state(State *s)
 {
 	int i;
+	char buf[PATH_MAX] = "\0";
+	uint64_t extsize = 0;
+	uint32_t numgrps = 0;
 
 	if (s->quiet)
 		return;
 
+	if (s->extent_alloc_size_in_clusters) {
+		numgrps = s->extent_alloc_size_in_clusters /
+			ocfs2_clusters_per_group(s->blocksize,
+						 s->cluster_size_bits);
+		extsize = (uint64_t)s->extent_alloc_size_in_clusters *
+			s->cluster_size;
+	}
+
+	ocfs2_snprint_feature_flags(buf, PATH_MAX, &s->feature_flags);
+
 	if (s->fs_type != FS_DEFAULT) {
 		for(i = 0; ocfs2_fs_types_table[i].ft_str; i++) {
 			if (ocfs2_fs_types_table[i].ft_type == s->fs_type) {
@@ -2446,21 +2535,24 @@ print_state(State *s)
 			}
 		}
 	}
-	printf("Filesystem label=%s\n", s->vol_label);
-	printf("Block size=%u (bits=%u)\n", s->blocksize, s->blocksize_bits);
-	printf("Cluster size=%u (bits=%u)\n", s->cluster_size, s->cluster_size_bits);
-	printf("Volume size=%"PRIu64" (%u clusters) (%"PRIu64" blocks)\n",
+	printf("Label: %s\n", s->vol_label);
+	printf("Features: %s\n", buf);
+	printf("Block size: %u (%u bits)\n", s->blocksize, s->blocksize_bits);
+	printf("Cluster size: %u (%u bits)\n", s->cluster_size, s->cluster_size_bits);
+	printf("Volume size: %"PRIu64" (%u clusters) (%"PRIu64" blocks)\n",
 	       s->volume_size_in_bytes, s->volume_size_in_clusters,
 	       s->volume_size_in_blocks);
-	printf("%u cluster groups (tail covers %u clusters, rest cover %u "
+	printf("Cluster groups: %u (tail covers %u clusters, rest cover %u "
 	       "clusters)\n", s->nr_cluster_groups, s->tail_group_bits,
 	       s->global_cpg);
+	printf("Extent allocator size: %"PRIu64" (%u groups)\n",
+	       extsize, numgrps);
 	if (s->hb_dev)
 		printf("Heartbeat device\n");
 	else
-		printf("Journal size=%"PRIu64"\n",
+		printf("Journal size: %"PRIu64"\n",
 		       s->journal_size_in_bytes);
-	printf("Initial number of node slots: %u\n", s->initial_slots);
+	printf("Node slots: %u\n", s->initial_slots);
 }
 
 static void
