@@ -193,6 +193,7 @@ struct _State {
 	int mount;
 	int no_backup_super;
 	int inline_data;
+	int dry_run;
 
 	uint32_t blocksize;
 	uint32_t blocksize_bits;
@@ -214,6 +215,8 @@ struct _State {
 	uint64_t journal_size_in_bytes;
 	int journal64;
 
+	uint32_t extent_alloc_size_in_clusters;
+
 	char *vol_label;
 	char *device_name;
 	unsigned char *uuid;
