@@ -17,6 +17,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <limits.h>
+#include <unistd.h>
 #include <inttypes.h>
 #include <errno.h>
 #include <syslog.h>
@@ -37,12 +38,14 @@
 struct service {
 	struct list_head	ms_list;
 	char			ms_service[PATH_MAX + 1];
+	int			ms_additional;	/* This is a second mount */
 };
 
 struct mountgroup {
 	struct list_head	mg_list;
 	struct cgroup		*mg_group;
 	int			mg_leave_on_join;
+	int			mg_registered;
 
 	char			mg_uuid[OCFS2_UUID_STR_LEN + 1];
 	char			mg_device[PATH_MAX + 1];
@@ -196,9 +199,9 @@ static struct service *find_service(struct mountgroup *mg,
 	return NULL;
 }
 
-static void remove_service(struct mountgroup *mg,
-			      const char *service)
+static void remove_service(struct mountgroup *mg, const char *service)
 {
+	int rc;
 	struct service *ms;
 
 	ms = find_service(mg, service);
@@ -220,6 +223,17 @@ static void remove_service(struct mountgroup *mg,
 		/* Set in-progress for leave */
 		mg->mg_ms_in_progress = ms;
 
+		if (mg->mg_registered) {
+			log_debug("Unregistering mountgroup %s",
+				  mg->mg_uuid);
+			rc = dlmcontrol_unregister(mg->mg_uuid);
+			if (rc)
+				log_error("Unable to deregister mountgroup "
+					  "%s: %s",
+					  mg->mg_uuid, strerror(-rc));
+			mg->mg_registered = 0;
+		}
+
 		log_debug("time to leave group %s", mg->mg_uuid);
 		if (mg->mg_group) {
 			log_debug("calling LEAVE for group %s",
@@ -275,6 +289,16 @@ static void add_service(struct mountgroup *mg, const char *device,
 				   mg->mg_uuid, service);
 			return;
 		}
+
+		/*
+		 * There can be more than one real mount.  However, if an
+		 * additional mount fails in sys_mount(2), we can't have
+		 * complete_mount() removing the service.  We only want that
+		 * to happen when it's the first mount.
+		 */
+		log_debug("Additional mount of %s starting for %s",
+			  mg->mg_uuid, service);
+		ms->ms_additional = 1;
 	} else {
 		ms = malloc(sizeof(struct service));
 		if (!ms) {
@@ -312,8 +336,42 @@ static void add_service(struct mountgroup *mg, const char *device,
 		list_add(&ms->ms_list, &mg->mg_services);
 }
 
+static void register_result(int status, void *user_data)
+{
+	struct mountgroup *mg = user_data;
+	struct service *ms;
+
+	if (!mg->mg_group) {
+		log_error("No cgroup (mg %s)", mg->mg_uuid);
+		return;
+	}
+
+	ms = mg->mg_ms_in_progress;
+	if (!ms) {
+		log_error("No service in progress for mountgroup %s",
+			  mg->mg_uuid);
+		return;
+	}
+
+	if (status) {
+		fill_error(mg, -status,
+			   "Error registering mg %s with dlm_controld: %s",
+			   mg->mg_uuid, strerror(-status));
+
+		/* remove_service() will kick off a LEAVE if needed */
+		remove_service(mg, ms->ms_service);
+		return;
+	}
+
+	log_debug("Mountgroup %s successfully registered with dlm_controld",
+		  mg->mg_uuid);
+	mg->mg_registered = 1;
+	notify_mount_client(mg);
+}
+
 static void finish_join(struct mountgroup *mg, struct cgroup *cg)
 {
+	int rc;
 	struct service *ms;
 
 	if (mg->mg_group) {
@@ -350,7 +408,19 @@ static void finish_join(struct mountgroup *mg, struct cgroup *cg)
 
 	/* Ok, we've successfully joined the group */
 	mg->mg_group = cg;
-	notify_mount_client(mg);
+
+	/* Now tell dlm_controld */
+	log_debug("Registering mountgroup %s with dlm_controld",
+		  mg->mg_uuid);
+	rc = dlmcontrol_register(mg->mg_uuid, register_result, mg);
+	if (rc) {
+		fill_error(mg, -rc,
+			   "Unable to register mountgroup %s with "
+			   "dlm_controld: %s",
+			   mg->mg_uuid, strerror(errno));
+		/* remove_service() will kick off a LEAVE if needed */
+		remove_service(mg, ms->ms_service);
+	}
 }
 
 static void mount_node_down(int nodeid, void *user_data)
@@ -364,20 +434,15 @@ static void mount_node_down(int nodeid, void *user_data)
 	if (err)
 		log_debug("%s while trying to send DOWN message",
 			  error_message(err));
-}
 
-static void force_node_down(int nodeid, void *user_data)
-{
-	struct mountgroup *mg = user_data;
-
-	log_error("Forcing node %d down in group %s", nodeid, mg->mg_uuid);
-	mount_node_down(nodeid, mg);
+	dlmcontrol_node_down(mg->mg_uuid, nodeid);
 }
 
 static void finish_leave(struct mountgroup *mg)
 {
 	struct list_head *p, *n;
 	struct service *ms;
+	struct timespec ts;
 
 	if (list_empty(&mg->mg_services) &&
 	    mg->mg_ms_in_progress) {
@@ -392,13 +457,33 @@ static void finish_leave(struct mountgroup *mg)
 		goto out;
 	}
 
-	/* This leave is unexpected */
-
+	/*
+	 * This leave is unexpected.  If we weren't part of the group, we
+	 * just cleanup our state.  However, if we were part of a group, we
+	 * cannot safely continue and must die.  Fail-fast allows other
+	 * nodes to make a decision about us.
+	 */
 	log_error("Unexpected leave of group %s", mg->mg_uuid);
-	if (mg->mg_group)
-		for_each_node(mg->mg_group, force_node_down, mg);
-	else
-		log_error("No mg_group for group %s", mg->mg_uuid);
+
+
+	if (mg->mg_group) {
+		log_error("Group %s is live, exiting", mg->mg_uuid);
+
+		/*
+		 * The _exit(2) may cause a reboot, and we want the errors
+		 * to hit syslogd(8).  We can't call sync(2) which might
+		 * sleep on an ocfs2 operation.  I'd say sleeping for 10ms
+		 * is a good compromise.  Local syslogd(8) won't have time
+		 * to write to disk, but a network syslogd(8) should get
+		 * the data.
+		 */
+		ts.tv_sec = 0;
+		ts.tv_nsec = 10000000;
+		nanosleep(&ts, NULL);
+		_exit(1);
+	}
+
+	log_error("No mg_group for group %s", mg->mg_uuid);
 
 	list_for_each_safe(p, n, &mg->mg_services) {
 		ms = list_entry(p, struct service, ms_list);
@@ -458,11 +543,11 @@ static void mount_set_group(struct cgroup *cg, void *user_data)
 /*
  * THIS FUNCTION CAUSES PROBLEMS.
  *
- * bail_on_mounts() is called when we are forced to exit via a signal or
- * cman dying on us.  As such, it tells ocfs2 that nodes are down but
- * not communicate with cman or cpg.  This can cause ocfs2 to self-fence or
- * cman to go nuts.  But hey, if you SIGKILL the daemon, you get what you
- * pay for.
+ * bail_on_mounts() is called when we are forced to exit via a signal
+ * or cluster stack dying on us.  As such, it tells ocfs2 that nodes
+ * are down but not communicate with the stack or cpg.  This can cause
+ * ocfs2 to self-fence or the stack to go nuts.  But hey, if you SIGKILL
+ * the daemon, you get what you pay for.
  */
 void bail_on_mounts(void)
 {
@@ -548,7 +633,7 @@ out:
 		send_message(fd, CM_STATUS, mg->mg_error, mg->mg_error_msg);
 		mg->mg_error = 0;
 
-		if (mg->mg_error == EALREADY)
+		if (rc == -EALREADY)
 			mg->mg_mount_notified = 1;
 		else {
 			log_error("mount: %s", mg->mg_error_msg);
@@ -651,14 +736,27 @@ int complete_mount(int ci, int fd, const char *uuid, const char *errcode,
 	 */
 	mg->mg_ms_in_progress = NULL;
 
+	if (ms->ms_additional) {
+		/*
+		 * Our additional real mount is done whether it succeeded
+		 * or failed.  We only have to clear the additional state
+		 * and reply OK.
+		 */
+		log_debug("Completed additional mount of filesystem %s, "
+			  "error is %ld",
+			  mg->mg_uuid, err);
+		ms->ms_additional = 0;
+		err = 0;
+	}
+
 	if (!err) {
 		mg->mg_mount_fd = -1;
 		mg->mg_mount_ci = -1;
 	} else {
 		/*
 		 * remove_service() will kick off a leave if this was
-		 * the last service.  As part of the leave, it will add
-		 * reset ms_in_progress.
+		 * the last service.  As part of the leave, it will set
+		 * ms_in_progress.
 		 */
 		remove_service(mg, service);
 
@@ -723,7 +821,30 @@ int remove_mount(int ci, int fd, const char *uuid, const char *service)
 	if (mg->mg_ms_in_progress) {
 		fill_error(&mg_error, EBUSY,
 			   "Another mount is in progress");
-		goto out;;
+
+		/*
+		 * If the service we're removing has ms_additional set, it
+		 * must be the filesystem service.  That means the
+		 * in_progress service is an additional real mount, but
+		 * the kernel is no longer mounted.
+		 *
+		 * As such, the in-progress service is now a new mount, and
+		 * we clear the ms_addditional flag.  It will succeed or
+		 * fail as a new mount.
+		 */
+		if (ms->ms_additional) {
+			if (ms != mg->mg_ms_in_progress)
+				log_error("Somehow ms_additional was set "
+					  "even though the in-progress "
+					  "mount isn't the filesystem "
+					  "(group %s, removing %s, "
+					  "in-progress %s",
+					  mg->mg_uuid, ms->ms_service,
+					  mg->mg_ms_in_progress->ms_service);
+			ms->ms_additional = 0;
+		}
+
+		goto out;
 	}
 
 	if ((mg->mg_mount_ci != -1) ||
@@ -773,6 +894,8 @@ void dead_mounter(int ci, int fd)
 	if (!ms)
 		return;
 
+	log_error("Mounter for filesystem %s, service %s died", mg->mg_uuid,
+		  ms->ms_service);
 	mg->mg_mount_ci = -1;
 	mg->mg_mount_fd = -1;
 
@@ -784,13 +907,27 @@ void dead_mounter(int ci, int fd)
 	if (list_empty(&ms->ms_list))
 		return;
 
+	/*
+	 * If this was just an additional real mount, we just clear the
+	 * state.
+	 */
+	if (ms->ms_additional) {
+		log_debug("Additional mounter of filesystem %s died",
+			  mg->mg_uuid);
+		ms->ms_additional = 0;
+		mg->mg_ms_in_progress = NULL;
+		return;
+	}
+
 	/*
 	 * We haven't notified the client yet.  Thus, the client can't have
 	 * called mount(2).  Let's just abort this service.  If this was
 	 * the last service, we'll plan to leave the group.
 	 */
-	if (!mg->mg_mount_notified)
+	if (!mg->mg_mount_notified) {
 		remove_service(mg, ms->ms_service);
+		return;
+	}
 
 	/*
 	 * XXX
@@ -799,9 +936,25 @@ void dead_mounter(int ci, int fd)
 	 * expecting the client to call mount(2).  But the client died.
 	 * We don't know if that happened, so we can't leave the group.
 	 *
-	 * That's not totally true, btw.  Eventually we'll be opening the
-	 * sysfs file.  Once we have that info, we'll do better here.
+	 * We do know, though, that all the other in-progress operations
+	 * (group join, dlmc_fs_register) must have completed, or we
+	 * wouldn't have set mg_mount_notified.  Thus, we can treat it as
+	 * a live mount.  Witness the power of a fully armed and
+	 * operational mountgroup.
+	 *
+	 * We can clear the in-progress flag and allow other mounters.  If
+	 * it really mounted, it can be unmounted.  If it didn't mount, the
+	 * state can be torn down with ocfs2_hb_ctl.
+	 *
+	 * Maybe later we'll learn how to detect the mount via the kernel
+	 * and tear it down ourselves.  But not right now.
 	 */
+
+	log_error("Kernel mount of filesystem %s already entered, "
+		  "assuming it succeeded",
+		  mg->mg_uuid);
+
+	mg->mg_ms_in_progress = NULL;
 }
 
 int send_mountgroups(int ci, int fd)
