@@ -75,6 +75,13 @@ struct tunefs_filesystem_state {
 	/* Non-zero if we've ever mucked with the allocator */
 	int		ts_allocation;
 
+	/*
+	 * Number of clusters in the filesystem.  If changed by a
+	 * resized filesystem, it is tracked here and used at final
+	 * close.
+	 */
+	uint32_t	ts_fs_clusters;
+
 	/* Size of the largest journal seen in tunefs_journal_check() */
 	uint32_t	ts_journal_clusters;
 
@@ -146,6 +153,7 @@ static errcode_t tunefs_set_state(ocfs2_filesys *fs)
 			s->ts_local_fd = -1;
 			s->ts_online_fd = -1;
 			s->ts_master = fs;
+			s->ts_fs_clusters = fs->fs_clusters;
 		} else
 			s = NULL;
 	}
@@ -447,7 +455,8 @@ errcode_t tunefs_empty_clusters(ocfs2_filesys *fs, uint64_t start_blk,
 	memset(buf, 0, io_blocks * fs->fs_blocksize);
 
 	while (total_blocks) {
-		ret = io_write_block(fs->fs_io, start_blk, io_blocks, buf);
+		ret = io_write_block_nocache(fs->fs_io, start_blk,
+					     io_blocks, buf);
 		if (ret)
 			goto bail;
 
@@ -498,7 +507,7 @@ static errcode_t tunefs_validate_inode(ocfs2_filesys *fs,
 		    strlen(OCFS2_INODE_SIGNATURE)))
 		return OCFS2_ET_BAD_INODE_MAGIC;
 
-	ocfs2_swap_inode_to_cpu(di, fs->fs_blocksize);
+	ocfs2_swap_inode_to_cpu(fs, di);
 
 	if (di->i_fs_generation != fs->fs_super->i_fs_generation)
 		return OCFS2_ET_INODE_NOT_VALID;
@@ -1087,8 +1096,25 @@ static errcode_t tunefs_open_bitmap_check(ocfs2_filesys *fs)
 	return tunefs_global_bitmap_check(fs);
 }
 
+void tunefs_update_fs_clusters(ocfs2_filesys *fs)
+{
+	struct tunefs_private *tp = to_private(fs);
+	struct tunefs_filesystem_state *state = tunefs_get_state(fs);
+
+	if (!(tp->tp_open_flags & TUNEFS_FLAG_ALLOCATION)) {
+		verbosef(VL_LIB,
+			 "Operation that claimed it would do no allocation "
+			 "just attempted to update the filesystem size\n");
+		return;
+	}
+
+	state->ts_fs_clusters = fs->fs_clusters;
+}
+
 static errcode_t tunefs_close_bitmap_check(ocfs2_filesys *fs)
 {
+	errcode_t ret;
+	uint32_t old_clusters;
 	struct tunefs_filesystem_state *state = tunefs_get_state(fs);
 
 	if (!state->ts_allocation)
@@ -1097,7 +1123,19 @@ static errcode_t tunefs_close_bitmap_check(ocfs2_filesys *fs)
 	if (state->ts_master != fs)
 		return 0;
 
-	return tunefs_global_bitmap_check(fs);
+	/*
+	 * An operation that resized the filesystem will have called
+	 * tunefs_update_fs_clusters().  The bitmap check needs this
+	 * new value, so we swap it in for the call.
+	 */
+	old_clusters = fs->fs_clusters;
+	fs->fs_clusters = state->ts_fs_clusters;
+	fs->fs_blocks = ocfs2_clusters_to_blocks(fs, fs->fs_clusters);
+	ret = tunefs_global_bitmap_check(fs);
+	fs->fs_clusters = old_clusters;
+	fs->fs_blocks = ocfs2_clusters_to_blocks(fs, fs->fs_clusters);
+
+	return ret;
 }
 
 static errcode_t tunefs_journal_check(ocfs2_filesys *fs)
@@ -1250,6 +1288,95 @@ static void tunefs_close_online_descriptor(ocfs2_filesys *fs)
 	}
 }
 
+/*
+ * If io_init_cache fails, we will go do the work without the
+ * io_cache, so there is no check for failure here.
+ */
+static void tunefs_init_cache(ocfs2_filesys *fs)
+{
+	errcode_t err;
+	struct tunefs_private *tp = to_private(fs);
+	struct tunefs_filesystem_state *state = tunefs_get_state(fs);
+	uint64_t blocks_wanted;
+	int scale_down;
+
+	/*
+	 * We have one I/O cache for all ocfs2_filesys structures.  This
+	 * guarantees a consistent view of the disk.  The master filesys
+	 * allocates it, child filesyses just use it.
+	 */
+	if (state->ts_master != fs) {
+		fs->fs_io = state->ts_master->fs_io;
+		return;
+	}
+
+	/*
+	 * Operations needing a large cache really want enough to
+	 * hold the whole filesystem in memory.  The rest of the
+	 * operations don't need much at all.  A cache big enough to
+	 * hold a chain allocator group should be enough.  Our largest
+	 * chain allocator is 4MB, so let's do 8MB and allow for
+	 * incidental blocks.
+	 */
+	if (tp->tp_open_flags & TUNEFS_FLAG_LARGECACHE)
+		blocks_wanted = fs->fs_blocks;
+	else
+		blocks_wanted = ocfs2_blocks_in_bytes(fs, 8 * 1024 * 1024);
+
+	/*
+	 * We don't want to exhaust memory, so we start with twice our
+	 * actual need.  When we find out how much we can get, we actually
+	 * get half that.
+	 */
+	blocks_wanted <<= 1;
+	scale_down = 1;
+
+	while (blocks_wanted > 0) {
+		io_destroy_cache(fs->fs_io);
+		verbosef(VL_LIB,
+			 "Asking for %"PRIu64" blocks of I/O cache\n",
+			 blocks_wanted);
+		err = io_init_cache(fs->fs_io, blocks_wanted);
+		if (!err) {
+			/*
+			 * We want to pin our cache; there's no point in
+			 * having a large cache if half of it is in swap.
+			 * However, some callers may not be privileged
+			 * enough, so once we get down to a small enough
+			 * number (512 blocks), we'll stop caring.
+			 */
+			err = io_mlock_cache(fs->fs_io);
+			if (err && (blocks_wanted <= 512))
+				err = 0;
+		}
+		if (!err) {
+			verbosef(VL_LIB, "Got %"PRIu64" blocks\n",
+				 blocks_wanted);
+			/* If we've already scaled down, we're done. */
+			if (!scale_down)
+				break;
+			scale_down = 0;
+		}
+
+		blocks_wanted >>= 1;
+	}
+}
+
+static void tunefs_drop_cache(ocfs2_filesys *fs)
+{
+	struct tunefs_filesystem_state *state = tunefs_get_state(fs);
+
+	/*
+	 * The master filesys created our cache.  We don't want
+	 * ocfs2_close() to kill it if we're closing a non-master,
+	 * so kill the pointer for those.
+	 */
+	if (state->ts_master == fs)
+		io_destroy_cache(fs->fs_io);
+	else
+		fs->fs_io = NULL;
+}
+
 static errcode_t tunefs_add_fs(ocfs2_filesys *fs, int flags)
 {
 	errcode_t err;
@@ -1370,10 +1497,14 @@ errcode_t tunefs_open(const char *device, int flags,
 	/*
 	 * We will use block cache in io.  Now, whether the cluster is
 	 * locked or the volume is mount local, in both situation we can
-	 * safely use cache.  If io_init_cache failed, we will go on the
-	 * tunefs work without the io_cache, so there is no check here.
+	 * safely use cache.  If we're not locked
+	 * (tunefs_special_errorp(err) != 0), we can't safely use it.
+	 * If this tunefs run has both special and regular operations,
+	 * ocfs2ne will retry with the regular arguments and will get
+	 * the cache for the regular operations.
 	 */
-	io_init_cache(fs->fs_io, ocfs2_extent_recs_per_eb(fs->fs_blocksize));
+	if (!err)
+		tunefs_init_cache(fs);
 
 	/*
 	 * SKIPCLUSTER operations don't check the journals - they couldn't
@@ -1402,6 +1533,7 @@ errcode_t tunefs_open(const char *device, int flags,
 out:
 	if (err && !tunefs_special_errorp(err)) {
 		if (fs) {
+			tunefs_drop_cache(fs);
 			tunefs_remove_fs(fs);
 			ocfs2_close(fs);
 			fs = NULL;
@@ -1431,6 +1563,7 @@ errcode_t tunefs_close(ocfs2_filesys *fs)
 		if (!err)
 			err = tmp;
 
+		tunefs_drop_cache(fs);
 		tunefs_remove_fs(fs);
 		tmp = ocfs2_close(fs);
 		if (!err)
